# -*- coding: utf-8 -*-

import os
import sys

import pytest
from . import TEST_DIR

from rdflib import Graph
from skosprovider_rdf.providers import RDFProvider

PY3 = sys.version_info[0] == 3

if PY3:  # pragma: no cover
    text_type = str
else:  # pragma: no cover
    text_type = unicode

@pytest.fixture(scope='module')
def products_provider():
    products_graph = Graph()
    abspath = os.path.abspath(TEST_DIR + "/data/simple_turtle_products")
    products_graph.parse(abspath, format="turtle")

    # Set up rdf_provider
    products_provider = RDFProvider(
        {'id': 'PRODUCTS'}, products_graph
    )
    return products_provider

@pytest.fixture(scope='module')
def trees_provider():
    trees_graph = Graph()
    abspath = os.path.abspath(TEST_DIR + "/data/trees.xml")
    trees_graph.parse(abspath, format="application/rdf+xml")
    trees_provider = RDFProvider(
        {'id': 'TREES'}, trees_graph
    )
    return trees_provider

@pytest.fixture(scope='module')
def materials_provider():
    import json

    materials_data = json.load(
        open(os.path.join(TEST_DIR, 'data', 'materiaal.txt')),
    )['materiaal']
    from skosprovider.providers import DictionaryProvider
    from skosprovider.uri import UriPatternGenerator
    from skosprovider.skos import ConceptScheme, Label, Note

    materials = DictionaryProvider(
        {'id': 'Materials'},
        materials_data,
        uri_generator=UriPatternGenerator('https://id.erfgoed.net/thesauri/materialen/%s'),
        conceptscheme=ConceptScheme(
            uri='https://id.erfgoed.net/thesauri/materialen',
            labels=[Label(type='prefLabel', language='nl', label='Materialen')],
            notes=[Note(type='scopeNote', language='nl', note='Materialen zijn grondstoffen of halfafgewerkte producten die vaak een rol spelen bij onroerend erfgoed.')]
        )
    )
    return materials



# -*- coding: utf-8 -*-
'''
This module contains utility functions for dealing with skos providers.
'''
from __future__ import unicode_literals
import logging
import sys

log = logging.getLogger(__name__)

from rdflib import Graph, Literal, Namespace
from rdflib.term import URIRef, BNode
from rdflib.namespace import RDF, SKOS, DCTERMS, VOID
SKOS_THES = Namespace('http://purl.org/iso25964/skos-thes#')
from skosprovider.skos import (
    Concept,
    Collection
)
from skosprovider.utils import (
    extract_language,
    add_lang_to_html
)

from xml.dom.minidom import Node, Element
import html5lib

PY3 = sys.version_info[0] == 3

if PY3:  # pragma: no cover
    binary_type = bytes
else:  # pragma: no cover
    binary_type = str


def rdf_dumper(provider):
    '''
    Dump a provider to a format that can be passed to a
    :class:`skosprovider.providers.RDFProvider`.

    :param skosprovider.providers.VocabularyProvider provider: The provider
        that wil be turned into an :class:`rdflib.graph.Graph`.

    :rtype: :class:`rdflib.graph.Graph`
    '''
    return _rdf_dumper(provider, None)


def rdf_c_dumper(provider, c):
    '''
    Dump one concept or collection from a provider to a format that can be passed to a
    :class:`skosprovider.providers.RDFProvider`.

    :param skosprovider.providers.VocabularyProvider provider: The provider
        that wil be turned into an :class:`rdflib.graph.Graph`.

    :param String c: identifier

    :rtype: :class:`rdflib.graph.Graph`
    '''
    return _rdf_dumper(provider, [c])


def _rdf_dumper(provider, id_list=None):
    '''
    Dump a provider to a format that can be passed to a
    :class:`skosprovider.providers.RDFProvider`.

    :param skosprovider.providers.VocabularyProvider provider: The provider
        that wil be turned into an :class:`rdflib.graph.Graph`.

    :param List id_list: List of id's of the data to dump.

    :rtype: :class:`rdflib.graph.Graph`
    '''
    graph = Graph()
    graph.namespace_manager.bind("skos", SKOS)
    graph.namespace_manager.bind("dcterms", DCTERMS)
    graph.namespace_manager.bind("skos-thes", SKOS_THES)
    graph.namespace_manager.bind("void", VOID)
    conceptscheme = URIRef(provider.concept_scheme.uri)
    _add_in_dataset(graph, conceptscheme, provider)
    graph.add((conceptscheme, RDF.type, SKOS.ConceptScheme))
    graph.add((conceptscheme, DCTERMS.identifier, Literal(provider.metadata['id'])))
    _add_labels(graph, provider.concept_scheme, conceptscheme)
    _add_notes(graph, provider.concept_scheme, conceptscheme)
    _add_sources(graph, provider.concept_scheme, conceptscheme)
    _add_languages(graph, provider.concept_scheme, conceptscheme)
    # Add triples using store's add method.
    if not id_list:
        id_list = [x['id'] for x in provider.get_all()]
        for c in provider.get_top_concepts():
            graph.add((conceptscheme, SKOS.hasTopConcept, URIRef(c['uri'])))
    for id in id_list:
        _add_c(graph, provider, id)

    return graph


def rdf_conceptscheme_dumper(provider):
    '''
    Dump all information of the conceptscheme of a provider to a format that can be passed to a
    :class:`skosprovider.providers.RDFProvider`.

    :param skosprovider.providers.VocabularyProvider provider: The provider
        that wil be turned into an :class:`rdflib.graph.Graph`.

    :rtype: :class:`rdflib.graph.Graph`
    '''
    graph = Graph()
    graph.namespace_manager.bind("skos", SKOS)
    graph.namespace_manager.bind("dcterms", DCTERMS)
    graph.namespace_manager.bind("skos-thes", SKOS_THES)
    graph.namespace_manager.bind("void", VOID)
    conceptscheme = URIRef(provider.concept_scheme.uri)
    _add_in_dataset(graph, conceptscheme, provider)
    graph.add((conceptscheme, RDF.type, SKOS.ConceptScheme))
    graph.add((conceptscheme, DCTERMS.identifier, Literal(provider.metadata['id'])))
    _add_labels(graph, provider.concept_scheme, conceptscheme)
    _add_notes(graph, provider.concept_scheme, conceptscheme)
    _add_sources(graph, provider.concept_scheme, conceptscheme)
    _add_languages(graph, provider.concept_scheme, conceptscheme)
    for c in provider.get_top_concepts():
        graph.add((conceptscheme, SKOS.hasTopConcept, URIRef(c['uri'])))

    return graph


def _add_in_dataset(graph, subject, provider):
    '''
    Checks if the provider says something about a dataset and if so adds
    void.inDataset statements.

    :param rdflib.graph.Graph graph: The graph to add statements to.
    :param rdflib.term.URIRef subject: The subject to add an inDataset statement to.
    :param skosprovider.providers.VocabularyProvider provider:
    '''

    duri = provider.get_metadata().get('dataset', {}).get('uri', None)
    if duri:
        graph.add((subject, VOID.inDataset, URIRef(duri)))


def _add_c(graph, provider, id):
    '''
    Adds a concept or collection to the graph.

    :param rdflib.graph.Graph graph: The graph to add statements to.
    :param skosprovider.providers.VocabularyProvider provider: Provider
    :param c: The id of a concept or collection.
    '''

    c = provider.get_by_id(id)
    subject = URIRef(c.uri)
    _add_in_dataset(graph, subject, provider)
    graph.add((subject, DCTERMS.identifier, Literal(c.id)))
    conceptscheme = URIRef(provider.concept_scheme.uri)
    graph.add((subject, SKOS.inScheme, conceptscheme))
    _add_labels(graph, c, subject)
    _add_notes(graph, c, subject)
    _add_sources(graph, c, subject)
    if isinstance(c, Concept):
        graph.add((subject, RDF.type, SKOS.Concept))
        for b in c.broader:
            broader = provider.get_by_id(b)
            if broader:
                graph.add((subject, SKOS.broader, URIRef(broader.uri)))
        for n in c.narrower:
            narrower = provider.get_by_id(n)
            if narrower:
                graph.add((subject, SKOS.narrower, URIRef(narrower.uri)))
        for r in c.related:
            related = provider.get_by_id(r)
            if related:
                graph.add((subject, SKOS.related, URIRef(related.uri)))
        for s in c.subordinate_arrays:
            subordinate_array = provider.get_by_id(s)
            if subordinate_array:
                graph.add((subject, SKOS_THES.subordinateArray, URIRef(subordinate_array.uri)))
        for k in c.matches.keys():
            for uri in c.matches[k]:
                graph.add((subject, URIRef(SKOS + k +'Match'), URIRef(uri)))
    elif isinstance(c, Collection):
        graph.add((subject, RDF.type, SKOS.Collection))
        for m in c.members:
            member = provider.get_by_id(m)
            if member:
                graph.add((subject, SKOS.member, URIRef(member.uri)))
        for s in c.superordinates:
            superordinate = provider.get_by_id(s)
            if superordinate:
                graph.add((subject, SKOS_THES.superOrdinate, URIRef(superordinate.uri)))


def _add_labels(graph, c, subject):
    for l in c.labels:
        predicate = URIRef(SKOS + l.type)
        lang = extract_language(l.language)
        graph.add((subject, predicate, Literal(l.label, lang=lang)))


def _add_notes(graph, c, subject):
    for n in c.notes:
        predicate = URIRef(SKOS + n.type)
        lang = extract_language(n.language)
        if n.markup is None:
            graph.add((subject, predicate, Literal(n.note, lang=lang)))
        else:
            html = add_lang_to_html(n.note, lang)
            graph.add((subject, predicate, Literal(html, datatype=RDF.HTML)))

def _add_sources(graph, c, subject):
    '''
    Add sources to the RDF graph.

    :param rdflib.graph.Graph graph: An RDF Graph.
    :param c: A :class:`skosprovider.skos.ConceptScheme`,
        :class:`skosprovider.skos.Concept` or :class:`skosprovider.skos.Collection`
    :param subject: The RDF subject to add the sources to.
    '''
    for s in c.sources:
        source = BNode()
        graph.add((source, RDF.type, DCTERMS.BibliographicResource))
        if s.markup is None:
            graph.add((source, DCTERMS.bibliographicCitation, Literal(s.citation)))
        else:
            graph.add((source, DCTERMS.bibliographicCitation, Literal(s.citation, datatype=RDF.HTML)))
        graph.add((subject, DCTERMS.source, source))

def _add_languages(graph, c, subject):
    '''
    Add languages to the RDF graph.

    :param rdflib.graph.Graph graph: An RDF Graph.
    :param c: A :class:`skosprovider.skos.ConceptScheme`.
    :param subject: The RDF subject to add the sources to.
    '''
    for l in c.languages:
        lang = extract_language(l)
        graph.add((subject, DCTERMS.language, Literal(l)))


def text_(s, encoding='latin-1', errors='strict'):
    """ If ``s`` is an instance of ``binary_type``, return
    ``s.decode(encoding, errors)``, otherwise return ``s``"""
    if isinstance(s, binary_type):
        return s.decode(encoding, errors)
    return s



# -*- coding: utf-8 -*-
import pytest
from . import TEST_DIR
import os
from rdflib import Graph
from rdflib.namespace import RDF, SKOS, DCTERMS
from rdflib.term import URIRef, Literal

from skosprovider.providers import DictionaryProvider
from skosprovider.skos import (
    ConceptScheme,
    Label,
    Note
)

from skosprovider_rdf.providers import RDFProvider
from skosprovider_rdf import utils

import logging
log = logging.getLogger(__name__)




@pytest.fixture(scope='module')
def tree_provider():
    larch = {
        'id': '1',
        'uri': 'http://id.trees.org/1',
        'type': 'concept',
        'labels': [
            {'type': 'prefLabel', 'language': 'en', 'label': 'The Larch'},
            {'type': 'prefLabel', 'language': 'nl', 'label': 'De Lariks'}
        ],
        'notes': [
            {
                'type': 'definition',
                'language': 'en',
                'note': 'A type of tree.'
            }, {
                'type': 'definition',
                'language': 'nl',
                'note': '<p>Een soort <srong>b</strong>oom.</p>',
                'markup': 'HTML'
            }
        ],
        'narrower': [],
        'broader': [],
        'related': [],
        'member_of': ['3'],
        'sources': [
            {'citation': 'Monthy Python. Episode Three: How to recognise different types of trees from quite a long way away.'}
        ]
    }

    chestnut = {
        'id': '2',
        'uri': 'http://id.trees.org/2',
        'type': 'concept',
        'labels': [
            {'type': 'prefLabel',
             'language': 'en',
             'label': 'The Chestnut'},
            {'type': 'altLabel',
             'language': 'nl',
             'label': 'De Paardekastanje'},
            {'type': 'altLabel',
             'language': 'fr',
             'label': 'la châtaigne'}
        ],
        'notes': [
            {
                'type': 'definition', 'language': 'en',
                'note': 'A different type of tree.'
            }
        ],
        'narrower': [],
        'broader': [],
        'related': [],
        'member_of': ['3'],
        'subordinate_arrays': [],
        'sources': [
            {
                'citation': '<strong>Monthy Python.</strong> Episode Three: How to recognise different types of trees from quite a long way away.',
                'markup': 'HTML'
            }
        ]
    }

    oak = {
        'id': '4',
        'uri': 'http://id.trees.org/4',
        'type': 'concept',
        'labels': [
            {'type': 'prefLabel',
             'language': 'en',
             'label': 'The Oak'},
            {'type': 'altLabel',
             'language': 'nl',
             'label': 'De Eik'},
            {'type': 'altLabel',
             'language': 'fr',
             'label': 'le chêne'}
        ],
        'notes': [
            {
                'type': 'definition', 'language': 'en',
                'note': 'An even differenter type of tree.'
            }
        ],
        'narrower': [],
        'broader': [],
        'related': [],
        'member_of': [],
        'subordinate_arrays': [],
        'matches': {
            'exact': ['https://id.erfgoed.net/thesauri/soorten/297']
        }
    }

    species = {
        'id': 3,
        'uri': 'http://id.trees.org/3',
        'labels': [
            {'type': 'prefLabel', 'language': 'en', 'label': 'Trees by species'},
            {'type': 'prefLabel', 'language': 'nl', 'label': 'Bomen per soort'}
        ],
        'type': 'collection',
        'members': ['1', '2', '4'],
        'member_of': [],
        'superordinates': []
    }

    tree_provider = DictionaryProvider(
        {
            'id': 'TREE',
            'dataset': {
                'uri': 'https://id.trees.org/dataset'
            }
        },
        [larch, chestnut, oak, species],
        concept_scheme=ConceptScheme(
            uri='http://id.trees.org',
            labels=[
                Label(
                    'Pythonic trees.',
                    type='prefLabel',
                    language='en'
                ),
                Label(
                    'Pythonische bomen.',
                    type='prefLabel',
                    language=None
                )
            ],
            notes=[
                Note(
                    '<p>Trees as used by Monthy Python.</p>',
                    type='definition',
                    language='en',
                    markup='HTML'
                )
            ]
        )
    )

    return tree_provider

class TestRDFDumperMaterials(object):

    def test_dump_dictionary_to_rdf(self, materials_provider):
        graph_dump = utils.rdf_dumper(materials_provider)
        xml = graph_dump.serialize(format='xml', encoding="UTF-8")
        if isinstance(xml, bytes):
            xml = xml.decode("UTF-8")
        assert '<?xml' == xml[:5]
        bont_skos_definition = '<skos:definition xml:lang="nl-BE">Bont is een gelooide dierlijke huid, dicht bezet met haren. Het wordt voornamelijk gebruikt voor het maken van kleding.</skos:definition>'
        dcterms_id_skos_definition = '<dcterms:identifier rdf:datatype="http://www.w3.org/2001/XMLSchema#integer">9</dcterms:identifier>'
        assert bont_skos_definition in xml
        assert dcterms_id_skos_definition in xml


class TestRDFDumperProducts(object):

    def test_dump_rdf_to_rdf(self, products_provider):
        graph_dump = utils.rdf_dumper(products_provider)
        xml = graph_dump.serialize(format='xml', encoding="UTF-8")
        if isinstance(xml, bytes):
            xml = xml.decode("UTF-8")
        assert '<?xml' == xml[:5]

        assert 'http://www.products.com/' in xml
        assert 'http://www.products.com/Jewellery' in xml
        assert 'http://www.products.com/Perfume' in xml
        assert 'http://www.products.com/Product' in xml
        assert 'http://www.products.com/Stuff' in xml

    def test_dump_rdf_compare_type(self, products_provider):
        graph_dump = utils.rdf_dumper(products_provider)
        assert isinstance(graph_dump, Graph)


class TestRDFDumperTrees(object):

    def test_dump_tree_to_rdf(self, tree_provider):
        graph_dump = utils.rdf_dumper(tree_provider)
        xml = graph_dump.serialize(format='xml', encoding="UTF-8")
        if isinstance(xml, bytes):
            xml = xml.decode("UTF-8")
        assert '<?xml' == xml[:5]

    def test_dump_larch_to_rdf(self, tree_provider):
        graph_dump = utils.rdf_c_dumper(tree_provider, 1)
        cs = URIRef('http://id.trees.org')
        assert (cs, RDF.type, SKOS.ConceptScheme) in graph_dump
        assert (cs, SKOS.definition, Literal('<p xml:lang="en">Trees as used by Monthy Python.</p>', datatype=RDF.HTML)) in graph_dump
        assert (cs, SKOS.prefLabel, Literal('Pythonic trees.', lang='en')) in graph_dump
        xml = graph_dump.serialize(format='xml', encoding="UTF-8")
        if isinstance(xml, bytes):
            xml = xml.decode("UTF-8")
        assert '<?xml' == xml[:5]

    def test_dump_chestnut_to_rdf(self, tree_provider):
        graph_dump = utils.rdf_c_dumper(tree_provider, 2)
        citations = list(graph_dump.objects(predicate=DCTERMS.bibliographicCitation))

        assert citations[0] == Literal(
            '<strong>Monthy Python.</strong> Episode Three: How to recognise different types of trees from quite a long way away.',
            datatype=RDF.HTML
        )

    def test_dump_oak_to_rdf(self, tree_provider):
        graph_dump = utils.rdf_c_dumper(tree_provider, 4)
        oak = URIRef('http://id.trees.org/4')
        eik = URIRef('https://id.erfgoed.net/thesauri/soorten/297')
        assert (oak, SKOS.exactMatch, eik) in graph_dump

    def test_dump_one_id_to_rdf_and_reload(self, tree_provider):
        graph_dump1 = utils.rdf_c_dumper(tree_provider, 1)
        provider = RDFProvider(
            {
                'id': 'Number1',
                'dataset': {
                    'uri': 'http://id.trees.org/dataset'
                }
            },
            graph_dump1
        )
        graph_dump2 = utils.rdf_c_dumper(provider, 1)
        graph_full_dump2 = utils.rdf_dumper(provider)
        assert len(graph_dump1) ==  len(graph_dump2)
        assert len(graph_full_dump2) > len(graph_dump2)

    def test_dump_conceptscheme_tree_to_rdf(self, tree_provider):
        graph_dump = utils.rdf_conceptscheme_dumper(tree_provider)
        cs = URIRef('http://id.trees.org')
        assert (cs, RDF.type, SKOS.ConceptScheme) in graph_dump
        assert (cs, SKOS.definition, Literal('<p xml:lang="en">Trees as used by Monthy Python.</p>', datatype=RDF.HTML)) in graph_dump
        assert (cs, SKOS.prefLabel, Literal('Pythonic trees.', lang='en')) in graph_dump



# -*- coding: utf-8 -*-

import os
import sys

import pytest
from . import TEST_DIR

from skosprovider.skos import Note, Collection, ConceptScheme
from skosprovider.utils import dict_dumper

from rdflib import Graph
from skosprovider_rdf.providers import RDFProvider

PY3 = sys.version_info[0] == 3

if PY3:  # pragma: no cover
    text_type = str
else:  # pragma: no cover
    text_type = unicode


class TestRDFProviderProducts(object):

    def test_get_vocabulary_id(self, products_provider):
        assert 'PRODUCTS' == products_provider.get_vocabulary_id()

    def test_conceptscheme(self, products_provider):
        cs = products_provider.concept_scheme

        assert cs.uri == 'http://www.products.com/Scheme'
        assert cs.label('en') is not None
        assert len(cs.languages) == 3

    def test_get_concept_by_id(self, products_provider):
        u_jewellery = "http://www.products.com/Jewellery"
        u_perfume = "http://www.products.com/Perfume"
        from skosprovider.skos import Concept

        con = products_provider.get_by_id(u_jewellery)
        assert isinstance(con, Concept)
        assert u_jewellery == con.id
        assert u_jewellery == con.uri
        assert u_perfume in con.related

    def test_get_unexisting_by_id(self, products_provider):
        con = products_provider.get_by_id(404)
        assert not con

    def test_createLabel(self, products_provider):
        with pytest.raises(ValueError):
            products_provider._create_label("literal","nonexistinglabeltype")

    def test_createNote(self, products_provider):
        with pytest.raises(ValueError):
            products_provider._create_note("literal","nonexistingnotetype")

    def test_get_concept_by_uri_equals_id(self, products_provider):
        u_product = "http://www.products.com/Product"
        cona = products_provider.get_by_id(u_product)
        conb = products_provider.get_by_uri(u_product)
        assert cona == conb

    def test_get_unexisting_by_uri(self, products_provider):
        con = products_provider.get_by_uri('http://www.products.com/Thingy')
        assert not con

    def test_concept_has_correct_note(self, products_provider):
        u_jewellery = "http://www.products.com/Jewellery"
        con = products_provider.get_by_id(u_jewellery)
        assert len(con.notes) == 2
        assert isinstance(con.notes[0], Note)

    def test_get_collection_by_id(self, products_provider):
        u_stuff = "http://www.products.com/Stuff"
        u_product = "http://www.products.com/Product"
        col = products_provider.get_by_id(u_stuff)
        assert isinstance(col, Collection)
        assert u_stuff == col.id
        assert u_product in col.members
        for m in col.members:
            m = products_provider.get_by_id(m)
            assert col.id in m.member_of

    def test_get_collection_by_uri_equals_id(self, products_provider):
        u_stuff = "http://www.products.com/Stuff"
        cola = products_provider.get_by_id(u_stuff)
        colb = products_provider.get_by_uri(u_stuff)
        assert cola.id == colb.id
        assert cola.uri == colb.uri

    def test_get_all(self, products_provider):
        all = products_provider.get_all()
        assert len(all) == 4

    def test_get_top_concepts(self, products_provider):
        all = products_provider.get_top_concepts()
        assert len(all) == 1

    def test_get_top_display(self, products_provider):
        all = products_provider.get_top_display()
        assert len(all) == 1

    def test_get_children_display_unexisting(self, products_provider):
        assert not products_provider.get_children_display(700)

    def test_get_children_display_collection(self, products_provider):
        u_stuff = "http://www.products.com/Stuff"
        children = products_provider.get_children_display(u_stuff)
        assert len(children) == 3

    def test_get_children_display_concept(self, products_provider):
        u_product = "http://www.products.com/Product"
        children = products_provider.get_children_display(u_product)
        assert len(children) == 2

    def test_find_all(self, products_provider):
        all = products_provider.find({})
        assert len(all) == 4

    def test_find_type_all(self, products_provider):
        all = products_provider.find({'type': 'all'})
        assert len(all) == 4

    def test_find_type_concept(self, products_provider):
        all = products_provider.find({'type': 'concept'})
        assert len(all) == 3

    def test_find_type_collection(self, products_provider):
        all = products_provider.find({'type': 'collection'})
        assert len(all) == 1

    def test_find_label_perfume(self, products_provider):
        all = products_provider.find({'label': 'Perfume'})
        assert len(all) == 1

    def test_find_label_perfume_type_concept(self, products_provider):
        all = products_provider.find({'label': 'Perfume', 'type': 'concept'})
        assert len(all) == 1

    def test_find_collection_unexisting(self, products_provider):
        with pytest.raises(ValueError):
            products_provider.find({'collection': {'id': 404}})

    def test_find_collection_stuff_no_depth(self, products_provider):
        u_stuff = "http://www.products.com/Stuff"
        all = products_provider.find({'collection': {'id': u_stuff}})
        assert len(all) == 3

    def test_expand_concept(self, products_provider):
        u_product = "http://www.products.com/Product"
        u_perfume = "http://www.products.com/Perfume"
        ids = products_provider.expand(u_product)
        assert u_perfume in ids

    def test_expand_collection(self, products_provider):
        u_stuff = "http://www.products.com/Stuff"
        u_perfume = "http://www.products.com/Perfume"
        ids = products_provider.expand(u_stuff)
        assert u_perfume in ids

    def test_expand_unexisting(self, products_provider):
        ids = products_provider.expand(404)
        assert not ids

    def test_no_literal(self, products_provider):
        assert products_provider._get_language_from_literal("test") is None


class TestMultipleConceptschemes(object):

    def test_pick_one_conceptscheme(self):
        wb_graph = Graph()
        abspath = os.path.abspath(TEST_DIR + "/data/waarde_en_besluit_types.ttl")
        wb_graph.parse(abspath, format="turtle")
        wb_provider = RDFProvider(
            {'id': 'WAARDETYPES'},
            wb_graph,
            concept_scheme_uri = 'https://id.erfgoed.net/thesauri/waardetypes'
        )
        assert 'https://id.erfgoed.net/thesauri/waardetypes' == wb_provider.concept_scheme.uri
        assert len(wb_provider.get_all()) == 21

    def test_set_a_conceptscheme_manually(self):
        wb_graph = Graph()
        abspath = os.path.abspath(TEST_DIR + "/data/waarde_en_besluit_types.ttl")
        wb_graph.parse(abspath, format="turtle")
        wb_provider = RDFProvider(
            {'id': 'WAARDETYPES'},
            wb_graph,
            concept_scheme = ConceptScheme(
                'https://id.erfgoed.net/thesauri/besluittypes'
            )
        )
        assert 'https://id.erfgoed.net/thesauri/besluittypes' == wb_provider.concept_scheme.uri
        assert len(wb_provider.get_all()) == 24

    def test_pick_wrong_conceptscheme(self):
        wb_graph = Graph()
        abspath = os.path.abspath(TEST_DIR + "/data/waarde_en_besluit_types.ttl")
        wb_graph.parse(abspath, format="turtle")
        with pytest.raises(RuntimeError) as exc:
            wb_provider = RDFProvider(
                {'id': 'WAARTYPES'},
                wb_graph,
                concept_scheme_uri = 'https://id.erfgoed.net/thesauri/waartypes'
            )
        assert 'https://id.erfgoed.net/thesauri/waardetypes' in str(exc.value)
        assert 'https://id.erfgoed.net/thesauri/besluittypes' in str(exc.value)

    def test_too_many_conceptscheme(self):
        toepassingen_graph = Graph()
        abspath = os.path.abspath(TEST_DIR + "/data/schemes.xml")
        toepassingen_graph.parse(abspath, format="application/rdf+xml")
        with pytest.raises(RuntimeError) as exc:
            toepassingen_provider = RDFProvider(
                {'id': 'TOEPASSINGEN'}, toepassingen_graph
            )
        assert 'https://id.erfgoed.net/toepassingen' in str(exc.value)
        assert 'https://id.erfgoed.net/applicaties' in str(exc.value)


class TestTreeProvider(object):

    def test_parse_without_conceptscheme_generates_default_uri(self, trees_provider):
        assert 'urn:x-skosprovider:trees' == trees_provider.concept_scheme.uri

    def test_parse_identifier(self, trees_provider):
        larch = trees_provider.get_by_id('1')
        assert larch.id == '1'

        chestnut = trees_provider.get_by_id('2')
        assert chestnut.id == '2'

        species = trees_provider.get_by_id(3)
        assert species.id == '3'

        assert not trees_provider.get_by_id('http://id.trees.org/1')
        assert not trees_provider.get_by_id('http://id.trees.org/2')
        assert not trees_provider.get_by_id('http://id.trees.org/3')

    def test_rdf_provider_list(self, trees_provider):
        dump = dict_dumper(trees_provider)

        assert len(dump) == 3
        chestnut = [item for item in dump if item['uri'] == 'http://id.trees.org/2'][0]
        assert chestnut['broader'] == []
        assert chestnut['id'] ==  '2'
        assert chestnut['member_of'] == ['3']
        assert chestnut['narrower'] == []
        label_en = [label for label in chestnut['labels'] if label['language'] == 'en'][0]
        assert label_en ==  {'label': 'The Chestnut', 'language': 'en', 'type': 'prefLabel'}
        label_nl = [label for label in chestnut['labels'] if label['language'] == 'nl'][0]
        assert label_nl == {'label': 'De Paardekastanje', 'language': 'nl', 'type': 'altLabel'}
        label_fr = [label for label in chestnut['labels'] if label['language'] == 'fr'][0]
        assert type(label_fr['label']) == text_type
        assert label_fr == {'label': u'la châtaigne', 'language': 'fr', 'type': 'altLabel'}
        assert {
                'language': 'en',
                'note': '<p>A different type of tree.</p>',
                'type': 'definition',
                'markup': 'HTML'
            } in chestnut['notes']
        assert {
                'language': 'und',
                'note': 'Een ander soort boom.',
                'type': 'definition',
                'markup': 'HTML'
            } in chestnut['notes']
        assert {
                'markup': 'HTML',
                'citation': '<strong>Monthy Python.</strong> Episode Three: How to recognise different types of trees from quite a long way away.'
        } in chestnut['sources']
        larch = [item for item in dump if item['uri'] == 'http://id.trees.org/1'][0]
        assert {
                'citation': 'Monthy Python. Episode Three: How to recognise different types of trees from quite a long way away.',
                'markup': None
            } in larch['sources']
        assert {
                'language': 'en',
                'note': 'A type of tree.',
                'type': 'definition',
                'markup': None
            } in larch['notes']
        assert {
                'language': 'nl',
                'note': '<p>Een soort boom.</p>',
                'type': 'definition',
                'markup': 'HTML'
            } in larch['notes']

    def test_find_matches_kastanjes(self, trees_provider):
        kastanjes = trees_provider.find({
            'matches': {'uri': 'https://id.erfgoed.net/thesauri/soorten/85'}
        })
        assert len(kastanjes) == 1

    def test_find_matches_no_related_larches(self, trees_provider):
        no_related_larches = trees_provider.find({
            'matches': {
                'uri': 'https://id.erfgoed.net/thesauri/soorten/666',
                'type': 'related'
            }
        })
        assert len(no_related_larches) == 0

    def test_find_matches_close_larches(self, trees_provider):
        close_larches = trees_provider.find({
            'matches': {
                'uri': 'https://id.erfgoed.net/thesauri/soorten/666',
                'type': 'close'
            }
        })
        assert len(close_larches) == 1



# -*- coding: utf-8 -*-

'''
This module contains an RDFProvider, an implementation of the
:class:`skosprovider.providers.VocabularyProvider` interface that uses a
:class:`rdflib.graph.Graph` as input.
'''

import logging
import rdflib
from rdflib.term import Literal, URIRef
from skosprovider_rdf.utils import text_

log = logging.getLogger(__name__)

from skosprovider.providers import MemoryProvider
from skosprovider.uri import (
    DefaultConceptSchemeUrnGenerator
)
from skosprovider.skos import (
    Concept,
    Collection,
    ConceptScheme,
    Label,
    Note,
    Source
)

from rdflib.namespace import RDF, SKOS, DC, DCTERMS
SKOS_THES = rdflib.Namespace('http://purl.org/iso25964/skos-thes#')

from language_tags import tags


class RDFProvider(MemoryProvider):

    '''
    Should the provider only take concepts into account explicitly linked
    to the conceptscheme?
    '''
    check_in_scheme = False

    '''
    A simple vocabulary provider that use an :class:`rdflib.graph.Graph`
    as input. The provider expects a RDF graph with elements that represent
    the SKOS concepts and collections.

    Please be aware that this provider needs to load the entire graph in memory.
    '''

    def __init__(self, metadata, graph, **kwargs):
        self.graph = graph
        self.check_in_scheme = False
        if not 'concept_scheme' in kwargs:
            kwargs['concept_scheme'] = self._cs_from_graph(metadata, **kwargs)
        else:
            self.check_in_scheme = True
        super(RDFProvider, self).__init__(metadata, [], **kwargs)
        self.list = self._from_graph()

    def _cs_from_graph(self, metadata, **kwargs):
        cslist = []
        for sub in self.graph.subjects(RDF.type, SKOS.ConceptScheme):
            uri = self.to_text(sub)
            cs = ConceptScheme(
                uri=uri,
                labels = self._create_from_subject_typelist(sub, Label.valid_types),
                notes = self._create_from_subject_typelist(sub, Note.valid_types),
                sources = self._create_sources(sub),
                languages = self._create_languages(sub)
            )
            cslist.append(cs)
        if len(cslist) == 0:
            return ConceptScheme(
                uri=DefaultConceptSchemeUrnGenerator().generate(
                    id=metadata.get('id')
                )
            )
        elif len(cslist) == 1:
            return cslist[0]
        else:
            if not 'concept_scheme_uri' in kwargs:
                raise RuntimeError(
                    'This RDF file contains more than one ConceptScheme. \
                    Please specify one. The following schemes were found: \
                    %s' % (", ".join([str(cs.uri) for cs in cslist]))
                )
            else:
                self.check_in_scheme = True
                csuri = kwargs['concept_scheme_uri']
                filteredcslist = [cs for cs in cslist if cs.uri == csuri]
                if len(filteredcslist) == 0:
                    raise RuntimeError(
                        'This RDF file contains more than one ConceptScheme. \
                        You specified an unexisting one. The following schemes \
                        were found: %s' % (", ".join([str(cs.uri) for cs in cslist]))
                    )
                else:
                    return filteredcslist[0]

    def _from_graph(self):
        clist = []
        for sub, pred, obj in self.graph.triples((None, RDF.type, SKOS.Concept)):
            if self.check_in_scheme and self._get_in_scheme(sub) != self.concept_scheme.uri:
                    continue
            uri = self.to_text(sub)
            matches = {}
            for k in Concept.matchtypes:
                matches[k] = self._create_from_subject_predicate(sub, URIRef(SKOS + k +'Match'))
            con = Concept(
                id = self._get_id_for_subject(sub, uri),
                uri=uri,
                concept_scheme = self.concept_scheme,
                labels = self._create_from_subject_typelist(sub, Label.valid_types),
                notes = self._create_from_subject_typelist(sub, Note.valid_types),
                sources = self._create_sources(sub),
                broader = self._create_from_subject_predicate(sub, SKOS.broader),
                narrower = self._create_from_subject_predicate(sub, SKOS.narrower),
                related = self._create_from_subject_predicate(sub, SKOS.related),
                member_of = [],
                subordinate_arrays = self._create_from_subject_predicate(sub, SKOS_THES.subordinateArray),
                matches = matches
            )
            clist.append(con)

        for sub, pred, obj in self.graph.triples((None, RDF.type, SKOS.Collection)):
            if self.check_in_scheme and self._get_in_scheme(sub) != self.concept_scheme.uri:
                    continue
            uri = self.to_text(sub)
            col = Collection(
                id=self._get_id_for_subject(sub, uri),
                uri=uri,
                concept_scheme = self.concept_scheme,
                labels = self._create_from_subject_typelist(sub, Label.valid_types),
                notes = self._create_from_subject_typelist(sub, (Note.valid_types)),
                sources = self._create_sources(sub),
                members = self._create_from_subject_predicate(sub, SKOS.member),
                member_of = [],
                superordinates = self._create_from_subject_predicate(sub, SKOS_THES.superOrdinate)
            )
            clist.append(col)
        self._fill_member_of(clist)
        return clist

    def _get_in_scheme(self, subject):
        '''
        Determine if a subject is part of a scheme.

        :param subject: Subject to get the sources for.
        :returns: A URI for the scheme a subject is part of or None if
            it's not part of a scheme.
        '''
        scheme = None
        scheme = self.graph.value(subject, SKOS.inScheme)
        if not scheme:
            scheme = self.graph.value(subject, SKOS.topConceptOf)
        return self.to_text(scheme) if scheme else None

    def _fill_member_of(self, clist):
        collections = list(set([c for c in clist if isinstance(c, Collection)]))
        for col in collections:
            for c in clist:
                if c.id in col.members:
                    c.member_of.append(col.id)
        return

    def _create_from_subject_typelist(self,subject,typelist):
        list = []
        for p in typelist:
            term=SKOS.term(p)
            list.extend(self._create_from_subject_predicate(subject,term))
        return list

    def _get_id_for_subject(self, subject, uri):
        if (subject, DCTERMS.identifier, None) in self.graph:
            return self.to_text(self.graph.value(subject=subject, predicate=DCTERMS.identifier, any=False))
        elif (subject, DC.identifier, None) in self.graph:
            return self.to_text(self.graph.value(subject=subject, predicate=DC.identifier, any=False))
        else:
            return uri

    def _create_from_subject_predicate(self, subject, predicate):
        list = []
        for s, p, o in self.graph.triples((subject, predicate, None)):
            type = predicate.split('#')[-1]
            if Label.is_valid_type(type):
                o = self._create_label(o, type)
            elif Note.is_valid_type(type):
                o = self._create_note(o, type)
            else:
                o = self._get_id_for_subject(o, self.to_text(o))
            list.append(o)
        return list

    def _create_label(self, literal, type):
        if not Label.is_valid_type(type):
            raise ValueError(
                'Type of Label is not valid.'
            )
        return Label(self.to_text(literal), type, self._get_language_from_literal(literal))

    def _read_markupped_literal(self, literal):
        if literal.datatype == RDF.HTML:
            df = literal.value.cloneNode(True)
            if df.firstChild and df.firstChild.attributes and 'xml:lang' in df.firstChild.attributes.keys():
                lang = self._scrub_language(df.firstChild.attributes.get('xml:lang').value)
                del df.firstChild.attributes['xml:lang']
            else:
                lang = 'und'
            return(df.toxml(), lang, 'HTML')
        else:
            return (literal, self._get_language_from_literal(literal), None)

    def _create_note(self, literal, type):
        if not Note.is_valid_type(type):
            raise ValueError(
                'Type of Note is not valid.'
            )
        l = self._read_markupped_literal(literal)
        return Note(self.to_text(l[0]), type, l[1], l[2])

    def _create_sources(self, subject):
        '''
        Create the sources for this subject.

        :param subject: Subject to get the sources for.
        :returns: A :class:`list` of :class:`skosprovider.skos.Source` objects.
        '''
        ret = []
        for s, p, o in self.graph.triples((subject, DCTERMS.source, None)):
            for si, pi, oi in self.graph.triples((o, DCTERMS.bibliographicCitation, None)):
                ret.append(
                    Source(
                        self.to_text(oi),
                        'HTML' if oi.datatype == RDF.HTML else None
                )
                )
        return ret

    def _create_languages(self, subject):
        '''
        Create the languages for this subject.

        :param subject: Subject to get the sources for.
        :returns: A :class:`list` of IANA language tags.
        '''
        ret = set()
        for s, p, o in self.graph.triples((subject, DCTERMS.language, None)):
            ret.add(self.to_text(self._scrub_language(o)))
        for s, p, o in self.graph.triples((subject, DC.language, None)):
            ret.add(self.to_text(self._scrub_language(o)))
        return ret

    def _scrub_language(self, language):
        if tags.check(language):
            return language
        else:
            log.warn('Encountered an invalid language %s. Falling back to "und".' % language)
            return 'und'

    def _get_language_from_literal(self, data):
        if not hasattr(data, 'language') or data.language is None:
            return None
        return self.to_text(self._scrub_language(data.language))

    def to_text(self, data):
        """
        data of binary type or literal type that needs to be converted to text.
        :param data
        :return: text representation of the data
        """
        return text_(data.encode('utf-8'), 'utf-8')



import os
import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()
with open(os.path.join(here, 'HISTORY.rst')) as f:
    HISTORY = f.read()

packages = [
    'skosprovider_rdf',
]

requires = [
    'skosprovider>=0.6.0',
    'rdflib',
    'html5lib'
]

setup(
    name='skosprovider_rdf',
    version='0.7.0',
    description='skosprovider_rdf',
    long_description=README + '\n\n' + HISTORY,
    long_description_content_type='text/x-rst',
    package_data={'': ['LICENSE']},
    package_dir={'skosprovider_rdf': 'skosprovider_rdf'},
    include_package_data=True,
    install_requires=requires,
    license='MIT',
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    author='Flanders Heritage Agency',
    author_email='ict@onroerenderfgoed.be',
    url='http://github.com/OnroerendErfgoed/skosprovider_rdf',
    keywords='rdf skos skosprovider vocabularies thesauri',
    packages=find_packages(),
    tests_require=requires,
    test_suite="skosprovider_rdf"
)
